#include <pspkernel.h>
#include <pspaudiocodec.h>
#include <pspaudio.h>
#include <pspmpeg.h>
#include "easymp3.h"

#ifdef PSP150
__attribute__ ((constructor))
void loaderInit(){
   pspKernelSetKernelPC();
   pspSdkInstallNoDeviceCheckPatch();
   pspSdkInstallNoPlainModuleCheckPatch();
   pspSdkInstallKernelLoadModulePatch();
}
#endif

static int bitrates[] = {0, 32, 40, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224,
 256, 320 };

unsigned long emp3CodecBuffer[65] __attribute__((aligned(64)));
short emp3MixBuffer[1152 * 2] __attribute__((aligned(64)));

short emp3OutputBuffer[4][1152 * 2] __attribute__((aligned(64)));
int emp3OutputIndex = 0;
static SceUID emp3Handle;

u8* mp3_data_buffer;
u16 mp3_data_align;
u16 mp3_channel_mode;
u32 emp3DataStart;
u32 emp3DataSize;
u8 mp3_getEDRAM;
const u32 mp3Channels=2;
const u32 mp3Samplerate=44100; //file's samplerate, also can be 48000,..
const u32 mp3SamplePerFrame=1152;

void emp3Init()
{
#ifdef PSP150
	int result = pspSdkLoadStartModule("flash0:/kd/me_for_vsh.prx", PSP_MEMORY_PARTITION_KERNEL);
	result = pspSdkLoadStartModule("flash0:/kd/videocodec.prx", PSP_MEMORY_PARTITION_KERNEL);
	result = pspSdkLoadStartModule("flash0:/kd/audiocodec.prx", PSP_MEMORY_PARTITION_KERNEL);
	result = pspSdkLoadStartModule("flash0:/kd/mpegbase.prx", PSP_MEMORY_PARTITION_KERNEL);
	result = pspSdkLoadStartModule("flash0:/kd/mpeg_vsh.prx", PSP_MEMORY_PARTITION_USER);

	pspSdkFixupImports(result);
#endif
	sceMpegInit();


}

void emp3Cleanup()
{

}

int emp3Load(const char *fname)
{
	emp3Handle=sceIoOpen(fname,PSP_O_RDONLY,0777);
	if ( emp3Handle<=0 )  {
		return 0;
	}
	emp3DataStart=sceIoLseek32(emp3Handle,0,PSP_SEEK_CUR);
	memset(emp3CodecBuffer, 0, sizeof(emp3CodecBuffer));

	emp3Playing=1;
	return 1;
}

void emp3Pause(int pause)
{
	emp3Paused=pause;
}

void emp3Stop()
{
	emp3StopRequested=1;
}

int emp3GetLength()
{
	return 1;
}

int emp3DetechID3(u8 * mp3_buffer_header)
{
	int eof=0;
      if(mp3_header_buf[0]=='I' && mp3_header_buf[1]=='D' && mp3_header_buf[2]==
'3' && mp3_header_buf[4]<0xff) {
              unsigned char id3_header_buf[6];
              if ( sceIoRead( mp3_handle, id3_header_buf, 6 ) != 6 ) {
              pspDebugScreenPrintf("Reading header failed\n");
                 eof = 1;
                 continue;
              }

              if(id3_header_buf[0]!=0xff && id3_header_buf[2]<0x80 && id3_header_buf[3]<0x80 && id3_header_buf[4]<0x80 && id3_header_buf[4]<0x80) {
                int size=(id3_header_buf[2]<<21)|(id3_header_buf[3]<<14)|(id3_header_buf[4]<<7)|id3_header_buf[5];
                mp3_data_start+=size+10;
                printf("ID3 header of 0x%x bytes detected\n",size+10);
                printf("Set mp3_data_start to 0x%x\n",mp3_data_start);

                sceIoLseek32(mp3_handle, mp3_data_start, PSP_SEEK_SET); //seek back
                if ( sceIoRead( mp3_handle, mp3_header_buf, 4 ) != 4 ) {
                  pspDebugScreenPrintf("Reading header failed\n");
                  eof = 1;
                  continue;
                }
              }
	return eof;
}

void emp3DecodeBlock()
{
int mp3_header = mp3_header_buf[0];
mp3_header = (mp3_header<<8) | mp3_header_buf[1];
mp3_header = (mp3_header<<8) | mp3_header_buf[2];
mp3_header = (mp3_header<<8) | mp3_header_buf[3];

int bitrate = (mp3_header & 0xf000) >> 12;
int padding = (mp3_header & 0x200) >> 9;

int frame_size = 144000*bitrates[bitrate]/mp3_samplerate + padding;

if ( mp3_data_buffer )
   free(mp3_data_buffer);
mp3_data_buffer = (u8*)memalign(64, frame_size);

sceIoLseek32(mp3_handle, mp3_data_start, PSP_SEEK_SET); //seek back
      if ( sceIoRead( mp3_handle, mp3_data_buffer, frame_size ) != frame_size )
{
         eof = 1;
      pspDebugScreenPrintf("Reading frame; ");
      pspDebugScreenPrintf("failed.\n");
         continue;
      }

      mp3_data_start += frame_size;

      mp3_codec_buffer[6] = (unsigned long)mp3_data_buffer;
      mp3_codec_buffer[8] = (unsigned long)mp3_mix_buffer;

      mp3_codec_buffer[7] = mp3_codec_buffer[10] = frame_size;
      mp3_codec_buffer[9] = mp3_sample_per_frame * 4;

      int res = sceAudiocodecDecode(mp3_codec_buffer, 0x1002);
      if ( res < 0 ) {
      pspDebugScreenPrintf("decode failed.\n");

         eof = 1;
         continue;
      }
      memcpy(mp3_output_buffer[mp3_output_index], mp3_mix_buffer, 1152*4);
      sceAudioOutputBlocking(audio_channel, PSP_AUDIO_VOLUME_MAX-1, mp3_output_b
uffer[mp3_output_index]);
      mp3_output_index = (mp3_output_index+1)%4;

      samplesdecoded = mp3_sample_per_frame;
}

int emp3Thread(void *private)
{
	if(sceAudiocodecCheckNeedMem(mp3_codec_buffer,0x1002)<0 ) return 0;
	if(sceAudiocodecGetEDRAM(mp3_codec_buffer,0x1002)<0 ) return 0;
	mp3_getEDRAM = 1;

	if(sceAudiocodecInit(mp3_codec_buffer,0x1002)<0) return 0;
	int audioChannel=sceAudioChReserve(0,emp3SamplePerFrame,PSP_AUDIO_FORMAT_STEREO);

	while(1) {
		if(!emp3Playing) {
			sceKernelDelayThread(1000000/60);
			continue;
		}
		int eof=0;
		while(!eof) {
			int samplesdecoded;
			memset(mp3_mix_buffer, 0, mp3_sample_per_frame*2*2);
			unsigned char mp3_header_buf[4];
			if ( sceIoRead( mp3_handle, mp3_header_buf, 4 ) != 4 ) {
				eof=1;
				continue;
			}
			emp3DetectHeader(mp3_header_buf);
			emp3DecodeBlock();
		}
		sceIoClose(emp3Handle);
		if(emp3DataBuffer) free(emp3DataBuffer);
		emp3DataBuffer=0;
	}
   if ( mp3_handle ) {
      sceIoClose(mp3_handle);
   }
   if ( mp3_data_buffer) {
      free(mp3_data_buffer);
   }
   if ( mp3_getEDRAM ) {
      sceAudiocodecReleaseEDRAM(mp3_codec_buffer);
   }

	return 1;
}
