#ifdef __cplusplus
extern "C" {
#endif

// image.c
typedef unsigned int Color;
typedef struct Image
{
        int textureWidth;  // the real width of data, 2^n with n>=0
        int textureHeight;  // the real height of data, 2^n with n>=0
        int imageWidth;  // the image width
        int imageHeight;	// the image height
        int isSwizzled;	// Is the image swizzled?
        int vram;		// Is the image in vram or not?
        Color* data;
} Image;
Image *loadPng(const char *filename);
void freeImage(Image *image);
void resetVRam();
extern int swizzleToVRam;
void swizzleFast(Image *source);
void saveImagePng(const char* filename, Color* data, int width, int height, int lineSize, int saveAlpha);
void saveImageTarga(const char* filename, Color* data, int width, int height, int lineSize, int saveAlpha);
enum FontId {
	FONT_HEADLINE,FONT_BODY,FONT_BODYHIGHLIGHT,FONT_MESSAGE,FONT_SMALL,FONT_SMALLHIGHLIGHT
};
void initFastFont();
void extentMessage(int *w,int *h, enum FontId fontId, const char *message);
void drawMessage(int x,int y, enum FontId fontId, const char *message);
void drawMessageAlpha(int x,int y, enum FontId fontId, const char *message,int alpha);
void drawMessageFormat(int x,int y,enum FontId fontId, const char *message);
Image *loadCell(const char *fname);
void drawCell(int x,int y,const char *fname);
void freeCells();
Image *newImage(int width,int height);

// julia.c
void generateVisualization(float a,float b,Image *image,int id);
void copyPalette(unsigned int *palette);

// undoredo
enum UndoRedoType { UR_SELECTION, UR_CHANGE, UR_ADD, UR_DELETE, UR_STEPANDREPEAT };

// md2fast.c
struct Vertex3DTP {
        float u,v;
        float x,y,z;
};
struct Vertex3DTPfast {
        short u,v;
        short x,y,z;
};

struct AnimFrame {
	float scale[3];
	float translate[3];
	struct Vertex3DTPfast *vert;
};
struct AnimMesh {
	Image *texture;
	int frameCount;
	int polyCount;
	int skinWidth,skinHeight;
	struct AnimFrame *frames;
};
struct AnimMesh *md2Load(const char *fname);
void md2GetFractionalFrame(struct AnimMesh *mesh,int frame,int nextFrame,float fraction,struct Vertex3DTP *buffer,float *minMax);
void md2Free(struct AnimMesh *mesh);

struct Model {
	char name[256];
	struct AnimMesh *mesh;
};


#ifdef __cplusplus
};
#endif
