#include <pspgu.h>
#include <pspsdk.h>
#include <psputility.h>
#include <pspdisplay.h>
#include <pspnet.h>
#include <pspnet_inet.h>
#include <pspnet_apctl.h>
#include <pspnet_resolver.h>
#include <psphttp.h>
#include <pspopenpsid.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/dirent.h>
#include <unistd.h>

#include "main.h"

int menuTimer;
int menuItem;
struct Options options;
enum GameMode pauseMode;
enum GameMode newPauseMode;

char alertMessage[256];
int menuMax;
struct MenuItem {
	enum MenuItemType {MI_ITEM,MI_TEXT,MI_TITLE,MI_LEFT} type;
	char *text;
};
struct MenuDirectory {
	struct MenuItem *item;
	int maxItem;	// Active selectable menu items
	int count;	// Total number of drawable items
};

struct MenuItem menuModeText[]={
	{MI_TITLE,"Resonate Main Menu"},
	{MI_ITEM,"New Game"},
	{MI_ITEM,"User Level"},
	{MI_ITEM,"How to Play"},
	{MI_ITEM,"High Scores"},
	{MI_ITEM,"Set Options"},
	{MI_ITEM,"Go Online"},
	{MI_ITEM,"Credits"},
	{MI_ITEM,"Quit"},
};
struct MenuItem pauseModeText[]={
	{MI_TITLE,"Pause Menu"},
	{MI_ITEM,"Continue"},
	{MI_ITEM,"How to Play"},
	{MI_ITEM,"Options"},
	{MI_ITEM,"Main Menu"},
};
struct MenuItem summaryModeText[]={
	{MI_TITLE,"Game Summary"},
	{MI_LEFT,"Score:                    "},
	{MI_LEFT,"Accuracy:                 "},
	{MI_LEFT,"Hit:                      "},
	{MI_LEFT,"Missed:                    "},	// Comment on performance.
	{MI_LEFT,"                            "},
	{MI_ITEM,"^ to Continue"},
};
struct MenuItem optionsModeText[]={
	{MI_TITLE,"Resonate Options"},
	{MI_ITEM,"Invert Y: Off                "},
	{MI_ITEM,"Targeting Speed: Medium      "},
	{MI_ITEM,"Music: Loud                  "},
	{MI_ITEM,"Sound Effects: ON            "},
	{MI_ITEM,"Clear High Scores"},
	{MI_ITEM,"Main Menu"},
};
struct MenuItem userLevelModeText[]={
	{MI_TITLE,"Resonate User Level"},
	{MI_ITEM,"a                                "},
	{MI_ITEM,"b                                "},
	{MI_ITEM,"c                                "},
	{MI_ITEM,"d                                "},
	{MI_ITEM,"e                                "},
	{MI_TEXT,"f                                "},
	{MI_ITEM,"More (Page 111 of 111)"},
	{MI_ITEM,"Main Menu"},
};
struct MenuItem onlineModeText[]={
	{MI_TITLE,"Resonate Online"},
	{MI_ITEM,"Upload High Scores"},
	{MI_ITEM,"Internet Ranking"},
	{MI_ITEM,"Upload User Level"},
	{MI_ITEM,"Download User Level"},
	{MI_ITEM,"Main Menu"},
};
struct MenuItem onlineHighScoresModeText[]={
	{MI_TITLE,"Resonate Online High Scores\0                "},
	{MI_LEFT,"0a                                                        "},
	{MI_LEFT,"1a                                                        "},
	{MI_LEFT,"2a                                                        "},
	{MI_LEFT,"3a                                                        "},
	{MI_LEFT,"4a                                                        "},
	{MI_LEFT,"5a                                                        "},
	{MI_ITEM,"More (Page 111 or 111)"},
	{MI_ITEM,"Main Menu"},
};
struct MenuItem highScoresModeText[]={
	{MI_TITLE,"Resonate High Scores"},
	{MI_TEXT,"0                          "},
	{MI_TEXT,"1                          "},
	{MI_TEXT,"2                          "},
	{MI_TEXT,"3                          "},
	{MI_TEXT,"4                          "},
	{MI_TEXT,"5                          "},
	{MI_ITEM,"More (Page 111 or 111)"},
	{MI_ITEM,"Main Menu"},
};
struct MenuItem instructionsModeText[]={
	{MI_TITLE,"How to Play Resonate"},
	{MI_LEFT,"Your goal is to evolve.  You start as a"},
	{MI_LEFT,"plankton, and eventually become a shark."},
	{MI_LEFT,"Collect Blue powerups to evolve. Red powerups"},
	{MI_LEFT,"are for chaos and require 8 hits. Use the "},
	{MI_LEFT,"analog stick to move the target. Hold X to target"},
	{MI_LEFT,"up to 8 enemies.  Press [] to unleash chaos."},
	{MI_LEFT,"START to pause. SELECT for screenshot."},
	{MI_LEFT,"L for extended looking. R for score menu."},
	{MI_LEFT,"^ for debug mode, with FPS."},
	{MI_ITEM,"Continue"},
};
struct MenuItem creditsModeText[]={
	{MI_TITLE,"Resonate Credits"},
	{MI_TEXT,"Design: hardhat, PSPdemon"},
	{MI_TEXT,"Programmer: hardhat"},
	{MI_TEXT,"Art: PSPdemon, meyitzo, Genz"},
	{MI_TEXT,"Music: DimensionT, Antti Mikkonnen"},
	{MI_TEXT,"Sound Effects: meanrabbit.com"},
	{MI_TEXT,"Testing: zion, Dutt, Wally, #psp-programming"},
	{MI_TEXT,"Inspired by Rez"},
	{MI_ITEM,"Continue"},
};
struct MenuDirectory menuDirectory[]={
	{menuModeText,8,9},
	{pauseModeText,4,5},
	{summaryModeText,1,7},
	{userLevelModeText,8,9},
	{optionsModeText,6,7},
	{onlineModeText,5,6},
	{highScoresModeText,2,9},
	{onlineHighScoresModeText,2,9},
	{instructionsModeText,1,11},
	{creditsModeText,1,9},
	{NULL,0,0},
};

char userLevelList[256][64];
int userLevelCount;
int userLevelFirst;
void showOptions();

struct HighScoreList {
	char fingerprint[64];
	char name[64];
	int score;
	int checkSum;
} highScoreList[256];
int highScoreCount;
int highScoreFirst;
void loadHighScores();
int findHighScore(const char *fingerprint);

int drawNetDialog();
int netDialog();
int netDialogActive=-1;
int httpTemplate;

int checkScore(const char *seed,int score)
{
	int i;
	int sum=0;
	for(i=0;i<strlen(seed);i++) {
		sum=(sum+sum)|(seed[i]&1);
	}
	return score^sum;
}

int findHighScore(const char *fingerprint)
{
	int i;
	for(i=0;i<highScoreCount;i++) {
		if(strcmp(fingerprint,highScoreList[i].fingerprint)==0) return i;
	}
	return -1;
}

void setMode(enum GameMode mode)
{
	//printf("Set mode to i%d->%d\n",gameMode,mode);
	if(mode==PAUSEMODE) {
		// We allocate all memory, and report the findings.
		void **first=0,*next=0;
		int available=0;
		int biggest=0;
		int size=16*1024*1024;	// Start at 16 meg.
		int chunks=0;
		while( size>1024) {
			first=malloc(size);
			if(!first) {
				size/=2;
				continue;
			}
			available+=size;
			if(size>biggest) biggest=size;
			chunks++;
			first[0]=next;
			next=first;
		}
		printf("Allocated %d bytes of memory in %d chunks.\n"
		"Biggest chunk of memory was %d\n",available,chunks,biggest);
		while(next) {
			first=next;
			next=first[0];
			free(first);
		}
	}
	
	newMode=mode;
	gameModeTimer=15+menuDirectory[gameMode].count*5;
	if(mode==SUMMARYMODE) {
		sprintf(summaryModeText[1].text,"Score: %d",gameScore);
		int outOf=enemyHit+enemyMiss;
		if(outOf==0) outOf=1;
		sprintf(summaryModeText[2].text,"Accuracy: %d%%",100*enemyHit/outOf);
		sprintf(summaryModeText[3].text,"Hits: %d of %d",enemyHit,enemyHit+enemyMiss);
		sprintf(summaryModeText[4].text,"Support: %d of %d",supportHit,supportHit+supportMiss);
		loadHighScores();
		int i=findHighScore(getCheckSum());
		int oldHigh=-1;
		int oldHighLine=-1;
		if(i>=0) {
			// old high score maybe?
			int newScore=highScoreList[i].score;
			int sum=checkScore(highScoreList[i].fingerprint,highScoreList[i].score);
			if(sum==highScoreList[i].checkSum) {
				oldHigh=newScore;
				oldHighLine=i;
				printf("Matched up old highscore of %d to item %d\n",newScore,i);
			} else {
				printf("Old high score doesn't match checksum\n");
				oldHighLine=i;
			}
		}
		if(gameScore>oldHigh) {
			if(oldHigh==-1) 
				sprintf(summaryModeText[5].text,"New highscore!");
			else
				sprintf(summaryModeText[5].text,"Beats %d",oldHigh);
			FILE *file=fopen("data/hiscore.txt","w");
			if(file) {
				for(i=0;i<highScoreCount;i++) {
					if(i==oldHighLine) {
						fprintf(file,"%s=%d@%d\"%s\n",getCheckSum(),gameScore,checkScore(getCheckSum(),gameScore),getName());
						printf("Updated %s=%d@%d\"%s\n",getCheckSum(),gameScore,checkScore(getCheckSum(),gameScore),getName());
					} else {
						if(strcasecmp(highScoreList[i].name,getName())!=0) {	// skip old versions of this level
							fprintf(file,"%s=%d@%d\"%s\n",highScoreList[i].fingerprint,highScoreList[i].score,highScoreList[i].checkSum,highScoreList[i].name);
							printf("Resaved %s=%d@%d\"%s\n",highScoreList[i].fingerprint,highScoreList[i].score,highScoreList[i].checkSum,highScoreList[i].name);
						} else {
							printf("Discarded: %s=%d@%d\"%s\n",highScoreList[i].fingerprint,highScoreList[i].score,highScoreList[i].checkSum,highScoreList[i].name);
						}
					}
				}
				if(oldHighLine<0) {
					fprintf(file,"%s=%d@%d\"%s\n",getCheckSum(),gameScore,checkScore(getCheckSum(),gameScore),getName());
				}
				fclose(file);
			}
		} else {
			sprintf(summaryModeText[5].text,"High score: %d",oldHigh);
		}
	}
	if(mode==PAUSEMODE) {
		pauseMode=PAUSEMODE;
		newPauseMode=PAUSEMODE;
	}
	if(newMode==OPTIONSMODE) {
		showOptions();
	}
}

void setPauseMode(enum GameMode mode)
{
	newPauseMode=mode;
	gameModeTimer=15+menuDirectory[pauseMode].count*5;
}

void drawMenu()
{
	int i,j;
	int x=0,y=0,w=0,h=0;
	struct MenuItem *item=menuDirectory[gameMode].item;
	int max=menuDirectory[gameMode].count;

	if(netDialogActive==0) {
		netDialogActive=drawNetDialog();
		return;
	};
	
	if(gameMode==PAUSEMODE) {
		item=menuDirectory[pauseMode].item;
		max=menuDirectory[pauseMode].count;
	}

	static Image *navhint=0;
	if(!navhint) navhint=loadPng("data/navhint.png");
	if(navhint) {
		drawSprite(0,0,navhint->imageWidth,navhint->imageHeight,navhint,480-navhint->imageWidth,272-navhint->imageHeight);
	}
	// Size dialog
	for(i=0;i<max;i++) {
		enum FontId font;
		switch(item[i].type) {
		default:
		case MI_ITEM: font=FONT_BODY; break;
		case MI_TEXT: font=FONT_MESSAGE; break;
		case MI_LEFT: font=FONT_MESSAGE; break;
		case MI_TITLE: font=FONT_HEADLINE; break;
		}
		extentMessage(&x,&y,font,item[i].text);
		h+=y-4;
		if(w<x) w=x;
	}
	if((gameMode==MENUMODE || gameMode==USERLEVELMODE) && newMode==GAMEMODE && menuTimer<15) {
		drawFilledRect(0,0,480,272,GU_RGBA(0,0,0,255-255*menuTimer/14));	// Fade scene to black
	}
	if(strlen(alertMessage)>0) {
		extentMessage(&x,&y,FONT_SMALL,alertMessage);
		h+=y;
		if(w<x) w=x;
	} else {
		h+=4;
	}
	w+=10;
	h+=10;
	x=480/2-w/2;
	y=272/2-h/2;
	if(menuTimer<15) {	// Animated box draw.
		w=w*menuTimer/14;
 		h=h*menuTimer/14;
	}
	// Text box
	drawFilledRect(x-5,y-5,w,h,GU_RGBA(0x00,0x00,0x00,0xa0));
	// Shadow
	drawFilledRect(x-5+w,y+5,10,h,GU_RGBA(0,0,0,0x60));
	drawFilledRect(x+5,y-5+h,w-10,10,GU_RGBA(0,0,0,0x60));
	// Outline
	drawFilledRect(x-5,y-5,1,h,GU_RGBA(224,217,73,0xff));
	drawFilledRect(x-5,y-5,w,1,GU_RGBA(224,217,73,0xff));
	drawFilledRect(x-5+w,y-5,1,h,GU_RGBA(224,217,73,0xff));
	drawFilledRect(x-5,y-5+h,w,1,GU_RGBA(224,217,73,0xff));
	// Draw Text
	j=0;
	for(i=0;i<max;i++) {
		int xx=0,yy=0;
		int ay;	// Animated y.
		int highlight=(item[i].type==MI_ITEM && j==menuItem);
		enum FontId font;
		switch(item[i].type) {
		default:
		case MI_ITEM: font=FONT_BODY; break;
		case MI_TEXT: font=FONT_MESSAGE; break;
		case MI_LEFT: font=FONT_MESSAGE; break;
		case MI_TITLE: font=FONT_HEADLINE; break;
		}
		extentMessage(&xx,&yy,font,item[i].text);
		if(gameModeTimer>0) {
			if(gameModeTimer<(max-i-1)*15/2+15) {
				ay=y+(max-i-1)*150+150*2-gameModeTimer*10*2;
			} else ay=y;
		} else if(menuTimer<i*7+15) {
			ay=y-i*150-150*2+menuTimer*10*2;
		} else ay=y;
		if(highlight) {
			int a=menuTimer%60;
			if(a>=30) a=60-a;
			a=a*159/30+96;
//			drawMessage(480/2-xx/2,ay,highlight==0?font:FONT_BODYHIGHLIGHT,item[i].text);
			sceGuTexFunc(GU_TFX_MODULATE, GU_TCC_RGBA);
			drawMessageAlpha(480/2-xx/2,ay,highlight==0?font:FONT_BODYHIGHLIGHT,item[i].text,a);
			sceGuTexFunc(GU_TFX_ADD, GU_TCC_RGBA);
			sceGuBlendFunc(GU_ADD, GU_SRC_ALPHA, GU_ONE_MINUS_SRC_ALPHA, 0, 0);
		} else {
			if(item[i].type==MI_LEFT) 
				drawMessage(x,ay,highlight==0?font:FONT_BODYHIGHLIGHT,item[i].text);
			else	
				drawMessage(480/2-xx/2,ay,highlight==0?font:FONT_BODYHIGHLIGHT,item[i].text);
		}

		y+=yy-4;
		if(item[i].type==MI_ITEM) j++;
	}
	if(strlen(alertMessage)>0) {
		int xx=0,yy=0;
		extentMessage(&xx,&yy,FONT_SMALL,alertMessage);
		drawMessage(480/2-xx/2,y,FONT_SMALL,alertMessage);
	}
	menuTimer+=3;
	if(gameModeTimer>0) menuTimer=gameModeTimer;
}

void updateUserLevelNames()
{
	int count=0;
	int i;
	for(i=0;i<6;i++) {
		if(i+userLevelFirst<userLevelCount) {
			char *s=userLevelList[i+userLevelFirst];
			if(strrchr(s,'/')) s=strrchr(s,'/')+1;
			strncpy(userLevelModeText[i+1].text,s,32);
			if(strrchr(userLevelModeText[i+1].text,'.')) strrchr(userLevelModeText[i+1].text,'.')[0]=0;
			userLevelModeText[i+1].type=MI_ITEM;
			count++;
		} else {
			strcpy(userLevelModeText[i+1].text,"- Empty Slot -");
			userLevelModeText[i+1].type=MI_TEXT;
		}
	}
	sprintf(userLevelModeText[7].text,"More (Page %d of %d)",userLevelFirst/6+1,(userLevelCount+5)/6);
	if(userLevelCount<1) {
		strcpy(userLevelModeText[7].text,"List empty");
	} else if(userLevelCount<=6) {
		strcpy(userLevelModeText[7].text,"List complete");
	}
	menuDirectory[USERLEVELMODE].maxItem=count+2;
}

void findUserLevels()
{
	DIR *dir=opendir("map");
	int i=0;
	struct dirent *ent;
	userLevelFirst=0;
	userLevelCount=0;
	
	if(!dir) {
		updateUserLevelNames();
		return;
	}
	
	while( (ent=readdir(dir))!=0 ) {
		if(ent->d_name[0]=='.') continue;
		if(strrchr(ent->d_name,'.')==0 || strcasecmp(strrchr(ent->d_name,'.'),".txt")!=0) continue;
		if(strncmp(ent->d_name,"level0",6)==0) continue;
		strcpy(userLevelList[i],"map/");
		strcat(userLevelList[i],ent->d_name);
		i++;
		if(i==256) break;
	}
	closedir(dir);
	userLevelCount=i;
	updateUserLevelNames();
}

void parseHighScores(const char *line)
{
	char idStr[256];
	if(strlen(line)>255) return;	// bad data.
	strcpy(idStr,line);

	if(highScoreCount==256) return;
	char *scoreStr=idStr,*checkStr=idStr,*nameStr=idStr;
	if(strchr(idStr,'\n')) strchr(idStr,'\n')[0]=0;
	scoreStr=strchr(idStr,'=');
	if(scoreStr) { scoreStr[0]=0; scoreStr++; }
	if(scoreStr) checkStr=scoreStr;
	if(!checkStr) checkStr=idStr;
	checkStr=strchr(checkStr,'@');
	if(checkStr) { checkStr[0]=0; checkStr++; }
	if(!checkStr) checkStr=idStr;
	nameStr=strchr(checkStr,'"');
	if(nameStr) { nameStr[0]=0; nameStr++; }
	if(!nameStr) nameStr="Unknown";
	int newScore=0;
	sscanf(scoreStr,"%d",&newScore);
	int check=0;
	sscanf(checkStr,"%d",&check);
	
	// Now place in data structure:
	strcpy(highScoreList[highScoreCount].name,nameStr);
	strcpy(highScoreList[highScoreCount].fingerprint,idStr);
	highScoreList[highScoreCount].checkSum=check;
	highScoreList[highScoreCount].score=newScore;
	printf("fingerprint: %s; score: %d; sum: %d; name: %s\n",highScoreList[highScoreCount].fingerprint,highScoreList[highScoreCount].score,highScoreList[highScoreCount].checkSum,highScoreList[highScoreCount].name);
	highScoreCount++;
}

void loadHighScores()
{
	FILE *file=fopen("data/hiscore.txt","r");
	highScoreCount=0;
	highScoreFirst=0;
	if(!file) return;
	
	printf("Opened old high score file!\n");
	char idStr[256];
	while(fgets(idStr,255,file) ) {
		parseHighScores(idStr);
		if(highScoreCount==256) break;
	}
	fclose(file);
	printf("Loaded %d high scores\n",highScoreCount);
}		

void loadOnlineHighScores(char *buffer)
{
	int max=strlen(buffer);
	highScoreCount=0;
	highScoreFirst=0;
	char *line;
	int i;
	int sol=0;	// start of line
	
	for(i=0;i<max;i++) {
		if(buffer[i]=='\r' || buffer[i]=='\n') {
			line=buffer+sol;
			buffer[i]=0;
			if(i>sol) {
				parseHighScores(line);
			}
			sol=i+1;
		}
	}
}

void updateHighScores()
{
	int i;
	for(i=0;i<6;i++) {
		if(highScoreFirst+i<highScoreCount) {
			sprintf(highScoresModeText[i+1].text,"%s: %d",highScoreList[highScoreFirst+i].name,highScoreList[highScoreFirst+i].score);
			printf("fingerprint: %s; score: %d; sum: %d\n",highScoreList[highScoreFirst+i].fingerprint,highScoreList[highScoreFirst+i].score,highScoreList[highScoreFirst+i].checkSum);
		} else {
			strcpy(highScoresModeText[i+1].text,"-");
		}
	}
	sprintf(highScoresModeText[7].text,"More (Page %d of %d)",highScoreFirst/6+1,(highScoreCount+5)/6);
	if(highScoreCount<1) {
		strcpy(highScoresModeText[7].text,"List empty");
	} else if(highScoreCount<=6) {
		strcpy(highScoresModeText[7].text,"List complete");
	}
}

void updateOnlineHighScores()
{
	int i;
	for(i=0;i<6;i++) {
		if(highScoreFirst+i<highScoreCount) {
			sprintf(onlineHighScoresModeText[i+1].text,"%d: %.20s %d",highScoreFirst+i+1,highScoreList[highScoreFirst+i].name,highScoreList[highScoreFirst+i].score);
			printf("fingerprint: %s; score: %d; sum: %d\n",highScoreList[highScoreFirst+i].fingerprint,highScoreList[highScoreFirst+i].score,highScoreList[highScoreFirst+i].checkSum);
		} else {
			strcpy(onlineHighScoresModeText[i+1].text,"-");
		}
	}
	sprintf(onlineHighScoresModeText[7].text,"More (Page %d of %d)",highScoreFirst/6+1,(highScoreCount+5)/6);
	if(highScoreCount<1) {
		strcpy(onlineHighScoresModeText[7].text,"List empty");
	} else if(highScoreCount<=6) {
		strcpy(onlineHighScoresModeText[7].text,"List complete");
	}
}

int handleMenu(int menuItem)
{
	switch(menuItem) {
	case 0:	// new game
		setMode(GAMEMODE);
		break;
	case 1:	// load game
		setMode(USERLEVELMODE);
		findUserLevels();
		break;
	case 2:	// how to play
		setMode(INSTRUCTIONSMODE);
		break;
	case 3: // high scores
		setMode(HIGHSCORESMODE);
		loadHighScores();
		updateHighScores();
		break;
	case 4:	// options
		setMode(OPTIONSMODE);
		break;
	case 5:	// online
#if _PSP_FW_VERSION >= 200
		if(sceKernelDevkitVersion()<0x02000010) {
			strcpy(alertMessage,"Can't in 1.5 folder of CFW");
			break;
		}
#endif
		setMode(ONLINEMODE);
		// Initalize networking.
        sceNetInit(128*1024, 42, 4*1024, 42, 4*1024);
        sceNetInetInit();
        sceNetApctlInit(0x8000, 48);
        sceNetResolverInit();
        if(!netDialog()) {
			setMode(MENUMODE);
			strcpy(alertMessage,"Could not access networking dialog!");
			sceNetResolverTerm();
			sceNetApctlTerm();
			sceNetInetTerm();
			sceNetTerm();
			break;
		}
		httpInit();
		break;
	case 6:	// credits
		setMode(CREDITSMODE);
		break;
	case 7:	// quit
		exitRequest=1;
		return 1;
	}
	return 0;
}

int handlePause(int menuItem)
{
	switch(menuItem) {
	case 0: // continue
		setMode(GAMEMODE);
		break;
	case 1: // how to play
		setPauseMode(INSTRUCTIONSMODE);
		break;
	case 2: // Options
		setPauseMode(OPTIONSMODE);
		break;
	case 3:	// Main menu
		setMode(MENUMODE);
		break;
	}
	return 0;
}

int handleUserLevel(int menuItem)
{
	if(menuItem<menuDirectory[USERLEVELMODE].maxItem-2) {
		setMode(GAMEMODE);
	} else if(menuItem==menuDirectory[USERLEVELMODE].maxItem-2) {
		userLevelFirst+=6;
		if(userLevelFirst>=userLevelCount) userLevelFirst=0;
		updateUserLevelNames();
		menuItem=menuDirectory[USERLEVELMODE].maxItem-2;
	} else {
		setMode(MENUMODE);
	}
	return 0;
}

int handleHighScores(int menuItem)
{
	if(menuItem==0) {
		highScoreFirst+=6;
		if(highScoreFirst>=highScoreCount) highScoreFirst=0;
		updateHighScores();
	} else {
		setMode(MENUMODE);
	}
	return 0;
}

int handleOnlineHighScores(int menuItem)
{
	if(menuItem==0) {
		highScoreFirst+=6;
		if(highScoreFirst>=highScoreCount) highScoreFirst=0;
		updateOnlineHighScores();
	} else {
		setMode(ONLINEMODE);
	}
	return 0;
}

void showOptions()
{
	sprintf(optionsModeText[1].text,"Invert Y: %s",options.invert?"On":"Off");
	sprintf(optionsModeText[2].text,"Targeting Accel: %d",(int)(options.speed*100));
	sprintf(optionsModeText[3].text,"Music Volume: %d%%",options.music*10);
	sprintf(optionsModeText[4].text,"Sound Effects: %s",options.sfx?"On":"Off");
}

void loadOptions()
{
	options.invert=0;
	options.music=7;
	options.sfx=1;
	options.speed=0.4f;
	FILE *file=fopen("data/options","rb");
	if(file) {
		fread(&options,1,sizeof(options),file);
		fclose(file);
	}
	if(options.music<0 || options.music>10) options.music=7;
	if(options.invert!=0) options.invert=1;
	if(options.sfx!=0) options.sfx=1;
	if(options.speed<0.3f || options.speed>0.6f) options.speed=0.4f;
	songVolume(options.music);
}

int handleOptions(int menuItem)
{
	switch(menuItem) {
	case 0:
		options.invert=options.invert?0:1;
		break;
	case 1:
		options.speed+=0.05f;
		if(options.speed>0.6f) options.speed=0.3f;
		break;
	case 2:
		options.music+=1;
		if(options.music>10) options.music=0;
		songVolume(options.music);
		break;
	case 3:
		options.sfx=options.sfx?0:1;
		break;
	case 4:
		unlink("data/hiscore.txt");
		strcpy(alertMessage,"High Score file reset.");
		return 0;
		break;
	case 5:
		if(gameMode==PAUSEMODE) setPauseMode(PAUSEMODE);
		else setMode(MENUMODE);
		return 0;
	}
	alertMessage[0]=0;
	FILE *file=fopen("data/options","wb");
	if(file) {
		fwrite(&options,1,sizeof(options),file);
		fclose(file);
	}
	showOptions();
	return 0;
}

extern unsigned int __attribute__((aligned(16))) gulist[262144];
static int stateLast=-1;
pspUtilityNetconfData data;
int netDialog()
{
#if _PSP_FW_VERSION >= 200
	memset(&data,0,sizeof(data));
	data.base.size = sizeof(data);
	data.base.language = PSP_SYSTEMPARAM_LANGUAGE_ENGLISH;
	data.base.buttonSwap = PSP_UTILITY_ACCEPT_CROSS;
	data.base.graphicsThread = 17;
	data.base.accessThread = 19;
	data.base.fontThread = 18;
	data.base.soundThread = 16;
	data.action = PSP_NETCONF_ACTION_CONNECTAP;

//#if  _PSP_FW_VERSION < 200
//		data.base.size=sizeof(pspUtilityNetconfData)-12;
//#endif
	netDialogActive=-1;
	int result=sceUtilityNetconfInitStart(&data);
	printf("sceUtilityNetconfInitStart: %08x\n",result);
	if(result<0) {
		data.base.size=sizeof(pspUtilityNetconfData)-12;
		result=sceUtilityNetconfInitStart(&data);
		printf("sceUtilityNetconfInitStart again: %08x\n",result);
		if(result<0) return 0;
	}
	netDialogActive=0;
#else
	netDialogActive=0;
	/* Connect using the first profile */
	int config=1;
	printf("Making connection to apctl\n");
	int err = sceNetApctlConnect(config);
	stateLast=-1;
	if(err!=0) {
	        printf("Resonate: sceNetApctlConnect returns %08X\n", err);
	        return -1;
	}
	printf("Resonate: Connecting...\n");
#endif

	return 1;
}

// returns -1 on quit, 0 on active, and 1 on success
int drawNetDialog()
{
	int done=0;
#if _PSP_FW_VERSION >= 200
	sceGuFinish();
	sceGuSync(0,0);
	
	switch(sceUtilityNetconfGetStatus()) {
	case PSP_UTILITY_DIALOG_NONE: printf("None\n"); break;
	case PSP_UTILITY_DIALOG_INIT: break;
	case PSP_UTILITY_DIALOG_VISIBLE: 
		sceUtilityNetconfUpdate(1);
		break;
	case PSP_UTILITY_DIALOG_QUIT:
		printf("NetDialog was quit.\n");
		sceUtilityNetconfShutdownStart(); 
		done=-1;
		break;	// cancelled??
	case PSP_UTILITY_DIALOG_FINISHED: 
		printf("NetDialog completed successfully.\n");
		sceUtilityNetconfShutdownStart(); 
		done=1;
		break;
	default: 
		printf("NetconfGetStatus: %08x\n",sceUtilityNetconfGetStatus());
		break;
	}
	
	sceGuStart(GU_DIRECT,gulist);
#else
	int state;
	int err=sceNetApctlGetState(&state);
	if(err!=0) {
		printf("Resonate: sceNetApctlGetState returns $%x\n",err);
		done=-1;
		return done;
	}
	char msg[256];
	sprintf(msg,"Connection state %d of 4", state);
	int xx=0,yy=0;
	extentMessage(&xx,&yy,FONT_SMALL,msg);
	drawMessage(480/2-xx/2,272/2-yy/2,FONT_SMALL,msg);

	if(state>stateLast) {
		printf("  connection state %d of 4\n", state);
		stateLast=state;
	}
	if(state==4) done=1;  // connected with static IP
	if(done==1) printf( "Resonate: Connected!\n");
#endif
	return done;
}

char *urlEncode(const char *msg)
{
	static char out[512];
	
	int i;
	int o=0;
	for(i=0;i<strlen(msg);i++) {
		if(msg[i]==' ') out[o++]='+';
		else if(msg[i]>126 || msg[i]=='+' || msg[i]=='=' || msg[i]=='&' || msg[i]=='?' || msg[i]<32) { sprintf(out+o,"%%%02x",msg[i]); o+=3; }
		else out[o++]=msg[i];
	}
	out[o]=0;
	
	return out;
}

int handleOnline(int menuItem)
{
	int result;
	static char httpResultBuf[81920];
	alertMessage[0]=0;
	static char postBody[10240];
	char url[256];
	int length;
	
	switch(menuItem) {
	case 0:	// Upload High Scores
		loadHighScores();
		if(highScoreCount>0) {
			int i;
			postBody[0]=0;
			char nickname[128];
			sceUtilityGetSystemParamString(PSP_SYSTEMPARAM_ID_STRING_NICKNAME, nickname, 128); 
			sprintf(postBody,"nickname=%s&psid=",nickname);
			PspOpenPSID psid;
			sceOpenPSIDGetOpenPSID(&psid);
			for(i=0;i<16;i++) sprintf(postBody+strlen(postBody),"%02x%s",psid.data[i],i==15?"":":");
			for(i=0;i<highScoreCount;i++) {
				sprintf(postBody+strlen(postBody),"&fingerprint%d=%s&score%d=%d&checksum%d=%d&name%d=%s",
				i,highScoreList[i].fingerprint,
				i,highScoreList[i].score,
				i,highScoreList[i].checkSum,
				i,urlEncode(highScoreList[i].name));
			}
			printf("Prepared to post: '%s'\n",postBody);
			length=8192;
			result=post("http://www.teamsushi.org/games/resonate/newscores.php",postBody,strlen(postBody),httpResultBuf,&length);
			if(result>0) {
				strncpy(alertMessage,httpResultBuf,80);
				alertMessage[80]=0;
				for(i=0;i<79;i++) if(alertMessage[i]>0 && alertMessage[i]<32) alertMessage[i]='/';
			}
		}
		break;
	case 1: // Internet Ranking
		length=81920;
		strcpy(onlineHighScoresModeText[0].text,"Ranking ");
		{
			const char *s=getName();
			if(strrchr(s,'\\')) s=strrchr(s,'\\')+1;
			if(strrchr(s,'/')) s=strrchr(s,'/')+1;
			strcpy(url,s);
			if(strrchr(url,'.')) strrchr(url,'.')[0]=0;
			url[32]=0;
		}
		strcat(onlineHighScoresModeText[0].text,url);
		sprintf(url,"http://www.teamsushi.org/games/resonate/levelrank.php?fingerprint=%s",getCheckSum());
		result=fetch(url,httpResultBuf,&length);
		if(result>0) {
			// parse the http results
			setMode(ONLINEHIGHSCORESMODE);
			
			loadOnlineHighScores(httpResultBuf);
			updateOnlineHighScores();
		}
		break;
	case 2: // Upload User Level
		// select level... upload
		strcpy(alertMessage,"Upload of levels not yet supported.");
		break;
	case 3: // Download User Level
		// get list of levels...select....download
		strcpy(alertMessage,"Download of levels not yet supported.");
		break;
	case 4: // Main Menu
		setMode(MENUMODE);
		// De-initialize networking
		httpTerm();
		sceNetResolverTerm();
		sceNetApctlTerm();
		sceNetInetTerm();
		sceNetTerm();
		break;
	}
	return 0;
}

int handleJoy(enum Buttons button,int up)
{
	if(netDialogActive==0) return 0;
	if(gameMode==GAMEMODE) {
		return handleJoyGame(button,up);
	}
	// MENUMODE or PAUSEMODE
	if(gameMode!=newMode || pauseMode!=newPauseMode) return 0;
	menuMax=menuDirectory[gameMode].maxItem;
	if(gameMode==PAUSEMODE) menuMax=menuDirectory[pauseMode].maxItem;
	if(up==1) return 0;
	switch(button) {
	case BT_UP:
		if(menuItem>0) menuItem--;
		else menuItem=menuMax-1;
		playSfx(SFX_MENUNEXT);
		break;
	case BT_DOWN:
		if(menuItem+1<menuMax) menuItem++;
		else menuItem=0;
		playSfx(SFX_MENUNEXT);
		break;
	case BT_RIGHT:
	case BT_CROSS:
	case BT_START:
		playSfx(SFX_MENUSELECT);
		if(gameMode==MENUMODE) return handleMenu(menuItem);
		else if(gameMode==PAUSEMODE) {
			if(pauseMode==OPTIONSMODE) return handleOptions(menuItem);
			else if(pauseMode!=PAUSEMODE) setPauseMode(PAUSEMODE);
			else return handlePause(menuItem);
		} else if(gameMode==USERLEVELMODE) return handleUserLevel(menuItem);
		else if(gameMode==OPTIONSMODE) return handleOptions(menuItem);
		else if(gameMode==SUMMARYMODE) break;
		else if(gameMode==ONLINEMODE) return handleOnline(menuItem);
		else if(gameMode==HIGHSCORESMODE) return handleHighScores(menuItem);
		else if(gameMode==ONLINEHIGHSCORESMODE) return handleOnlineHighScores(menuItem);
		else setMode(MENUMODE);
		break;
	case BT_TRIANGLE:
		if(gameMode==SUMMARYMODE) {
			playSfx(SFX_MENUSELECT);
			setMode(MENUMODE);
		}
		break;
	default:
		break;
	}
	return 0;
}
