/* ripple effect inspired by 
 * http://freespace.virgin.net/hugo.elias/graphics/x_water.htm
 */

#include <pspkernel.h>
#include <pspgu.h>
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include "main.h"

const int rippleWidth=480, rippleHeight=272;
short *ripple1,*ripple2;
Image *rippleImage;
long rippleFrequency;
long rippleDelay;

SceKernelUtilsMt19937Context ctx;



void rippleInit()
{
	ripple1=(short *)calloc( rippleWidth*rippleHeight,sizeof(short));
	ripple2=(short *)calloc( rippleWidth*rippleHeight,sizeof(short));
	rippleFrequency=256;
	rippleDelay=0;
	rippleImage=newImage(rippleWidth,rippleHeight);
	sceKernelUtilsMt19937Init(&ctx, time(NULL));
}


void rippleTerm()
{
	if(ripple1) free(ripple1);
	if(ripple2) free(ripple2);
	if(rippleImage) freeImage(rippleImage);
	ripple1=0;
	ripple2=0;
}

extern void *drawBuffer;
void rippleUpdate(long elapsed)
{
	rippleDelay-=elapsed;
	if(rippleDelay<0) {
		if(rippleFrequency>32) rippleFrequency-=16;
		else rippleFrequency-=4;

		if(rippleFrequency<1) rippleFrequency=1;
	}
	while(rippleDelay<0) {
		// Time for a new ripple to be added.
		int x=sceKernelUtilsMt19937UInt(&ctx)%rippleWidth;
		int y=sceKernelUtilsMt19937UInt(&ctx)%rippleHeight;
		ripple2[x+y*rippleWidth]=512;
		printf("New riple at x=%d,y=%d\n",x,y);
		rippleDelay+=rippleFrequency;
	}
	// Now spread the high points around.
	int k;
	for(k=0;k<2;k++) {
		int i,j;
		for(j=1;j<rippleHeight-1;j++) {
			for(i=1;i<rippleWidth-1;i++) {
				int pos=j*rippleWidth+i;
				ripple1[pos]=((ripple2[pos-1] + ripple2[pos+1] + ripple2[pos-rippleWidth] + ripple2[pos+rippleWidth])>>1) - ripple1[pos];
				ripple1[pos] -= ripple1[pos]>>5;
			}
		}
		// Now swap buffers.
		short *temp;
		temp=ripple1;
		ripple1=ripple2;
		ripple2=temp;
	}
}

void rippleDraw(Image *image)
{
	if(!image) {
		printf("No ripple source texture.\n");
		return;
	}

	int i,j;

	for(j=0;j<rippleHeight;j++) {
		for(i=0;i<rippleWidth;i++) {
			int offx=0;
			int offy=0;
			int pos=i+j*rippleWidth;
			if(i>0 && i<rippleWidth-2) offx=ripple1[pos-1]-ripple1[pos+1];
			if(j>0 && j<rippleHeight-2) offy=ripple1[pos-rippleWidth]-ripple1[pos+rippleWidth];
			int x=i+offx,y=j+offy;
			if(x<0) x=0;
			if(x>=image->imageWidth) x=image->imageWidth-1;
			if(y<0) y=0;
			if(y>=image->imageHeight) y=image->imageHeight-1;
			rippleImage->data[i+j*rippleImage->textureWidth]=image->data[x+image->textureWidth*y];
			//if(offx || offy) printf("i=%d,j=%d <- x=%d,y=%d (off %d,%d) %08x\n",i,j,x,y,offx,offy,rippleImage->data[i+j*rippleImage->textureWidth]);
		}
	}
	// Now render that to the screen
	sceKernelDcacheWritebackAll();
	//drawSprite(0,0,rippleWidth,rippleHeight,rippleImage,0,0);
        sceGuCopyImage(GU_PSM_8888, 0, 0, rippleImage->imageWidth, rippleImage->imageHeight, rippleImage->textureWidth, rippleImage->data, 0, 0, 512, (Color *)((unsigned int)drawBuffer+0x4000000));
}

